<?php
/**
 * ZDbConnection change the dbConnection by changing CDbCommandBuilder for repair a bug in framework
 * For use this class the dbConnection should be set with class=ZDbConnection
 */
Yii::import('application.extensions.PHPPDO.CPdoDbConnection');
class ZDbConnection extends CPdoDbConnection
{
	protected $_schema;
	public $pdoClass='PHPPDO';
	/**
	 * @return CDbSchema the database schema for the current connection
	 * @throws CException if the connection is not active yet
	 */
	public function getSchema()
	{
		if($this->_schema!==null)
			return $this->_schema;
		else
		{
			if(!$this->getActive())
				throw new CDbException('CDbConnection is inactive and cannot perform any DB operations.');
			$driver=$this->getDriverName();
			switch(strtolower($driver))
			{
				case 'pgsql':  // PostgreSQL
					return $this->_schema=new CPgsqlSchema($this);
				case 'mysqli': // MySQL
				case 'mysql':
					return $this->_schema=new ZMysqlSchema($this);
				case 'sqlite': // sqlite 3
				case 'sqlite2': // sqlite 2
					return $this->_schema=new CSqliteSchema($this);
				case 'mssql': // Mssql driver on windows hosts
				case 'dblib': // dblib drivers on linux (and maybe others os) hosts
					return $this->_schema=new CMssqlSchema($this);
				case 'oci':  // Oracle driver
					return $this->_schema=new COciSchema($this);
				case 'ibm':
				default:
					throw new CDbException('CDbConnection does not support reading schema for {driver} database.');
			}
		}
	}

}



class ZMysqlSchema extends CMysqlSchema
{
	/**
	 * Creates a command builder for the database.
	 * This method may be overridden by child classes to create a DBMS-specific command builder.
	 * @return CDbCommandBuilder command builder instance
	 */
	protected function createCommandBuilder()
	{
		return new ZDbCommandBuilder($this);
	}

}


class ZDbCommandBuilder extends CDbCommandBuilder
{

	/**
	 * Creates a COUNT(*) command for a single table.
	 * @param mixed the table schema ({@link CDbTableSchema}) or the table name (string).
	 * @param CDbCriteria the query criteria
	 * @param string the alias name of the primary table. Defaults to 't'.
	 * @return CDbCommand query command.
	 */
	public function createCountCommand($table,$criteria,$alias='t')
	{

		$this->ensureTable($table);
		$select=is_array($criteria->select) ? implode(', ',$criteria->select) : $criteria->select;
		if($criteria->alias!='')
			$alias=$criteria->alias;
		$alias=$this->getschema()->quoteTableName($alias);
		$sql=($criteria->distinct ? 'SELECT DISTINCT':'SELECT')." {$select} FROM {$table->rawName} $alias";
		$sql=$this->applyJoin($sql,$criteria->join);
		$sql=$this->applyCondition($sql,$criteria->condition);
		$sql=$this->applyGroup($sql,$criteria->group);
		$sql=$this->applyHaving($sql,$criteria->having);
		
		$sql="SELECT COUNT(*) FROM ($sql) sq";
		$command=$this->dbConnection->createCommand($sql);
		$this->bindValues($command,$criteria->params);
		return $command;
	}

}